/***************************************************************************
 *   Copyright (C) 2003 by Moris Ravasio                                   *
 *   moris.r@tin.it                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


#include "kink.h"

extern "C" {
	#include <inklevel.h>
}

KInk::KInk(QWidget* parent, const char* name, WFlags fl)
    : MainDlg(parent,name,fl)
{
}

KInk::~KInk()
{
}

/*$SPECIALIZATION$*/
void KInk::btnUpdate_clicked()
{
	update();
}

void KInk::slotCloseClicked()
{
	close();
}

void KInk::btnAbout_clicked()
{
	KAboutApplication about(this, "aboutDlg", true);
	about.exec();
}

void KInk::cmbDevices_activated(const QString&)
{
	currDeviceIdx = cmbDevices->currentItem();
	update();
}



#include "kink.moc"



/*!
    \fn KInk::update(void)
 */
void KInk::update(void)
{
	int callback, p_type, p_idx;
	struct ink_level *level;

	// Disable all progress bars preventively
	barBlack->setEnabled(false);
	barColor->setEnabled(false);
	barCyan->setEnabled(false);
	barMagenta->setEnabled(false);
	barYellow->setEnabled(false);
	barPhoto->setEnabled(false);
	barPhotoCyan->setEnabled(false);
	barPhotoMagenta->setEnabled(false);
	barPhotoYellow->setEnabled(false);
	barBlack->setProgress(0);
	barColor->setProgress(0);
	barCyan->setProgress(0);
	barMagenta->setProgress(0);
	barYellow->setProgress(0);
	barPhoto->setProgress(0);
	barPhotoCyan->setProgress(0);
	barPhotoMagenta->setProgress(0);
	barPhotoYellow->setProgress(0);

	frmErrorMsg->hide();

	p_type = (currDeviceIdx < MAX_PARPORT ? PARPORT : USB);
	p_idx = (currDeviceIdx < MAX_PARPORT ? currDeviceIdx : currDeviceIdx-MAX_PARPORT);

	level = new struct ink_level();
	if(level==NULL)
	{
		showError(tr2i18n("<b>Cannot update information</b><br><br>The system if probably run out of memory resources. Recover a little quantity of memory closing some applications, and click on <b>Update</b>"));

		return;
	}

	callback=get_ink_level(p_type,p_idx,level);
	if(callback==ERROR || level->type==NO_INK_LEVEL_FOUND)
	{
		showError(tr2i18n("<b>Cannot read ink level from specified device</b><br><br>An exception as occoured in <b>libinklevel</b> library: your printer is not in the list of supported devices or there is no printer plugged on the specified port."));

		delete level;
		return;
	}

	if(level->color == level->black == level->cyan == level->magenta == level->yellow ==
			level->photo == level->photocyan == level->photomagenta == level->photoyellow == 0)
	{
		showError(tr2i18n("<b>Device not ready?</b><br><br>KInk can't read the ink level from your device. Ensure that your printer is plugged and switched on and check the correct ink cartdrige installation.<br><br>Click on <b>Update</b> to try again"));

		delete level;
		return;
	}

	switch (level->type) {
  case ONE_COLOR_FOUND:
		barColor->setEnabled(true);
		barColor->setProgress(level->color);
    break;
  case ONE_BLACK_COLOR_FOUND:
		barBlack->setEnabled(true);
		barBlack->setProgress(level->black);
    break;
  case TWO_COLORS_FOUND:
		barBlack->setEnabled(true);
		barColor->setEnabled(true);
		barBlack->setProgress(level->black);
		barColor->setProgress(level->color);
    break;
  case TWO_PHOTO_COLORS_FOUND:
		barPhoto->setEnabled(true);
		barColor->setEnabled(true);
		barPhoto->setProgress(level->photo);
		barColor->setProgress(level->color);
    break;
  case THREE_COLORS_FOUND:
		barCyan->setEnabled(true);
		barMagenta->setEnabled(true);
		barYellow->setEnabled(true);
		barCyan->setProgress(level->cyan);
		barMagenta->setProgress(level->magenta);
		barYellow->setProgress(level->yellow);
    break;
  case FOUR_COLORS_FOUND:
		barBlack->setEnabled(true);
		barCyan->setEnabled(true);
		barMagenta->setEnabled(true);
		barYellow->setEnabled(true);
		barBlack->setProgress(level->black);
		barCyan->setProgress(level->cyan);
		barMagenta->setProgress(level->magenta);
		barYellow->setProgress(level->yellow);
    break;
  case SIX_COLORS_FOUND:
		barBlack->setEnabled(true);
		barPhotoCyan->setEnabled(true);
		barPhotoMagenta->setEnabled(true);
		barCyan->setEnabled(true);
		barMagenta->setEnabled(true);
		barYellow->setEnabled(true);
		barBlack->setProgress(level->black);
		barPhotoCyan->setProgress(level->photocyan);
		barPhotoMagenta->setProgress(level->photomagenta);
		barCyan->setProgress(level->cyan);
		barMagenta->setProgress(level->magenta);
		barYellow->setProgress(level->yellow);
    break;
  case SEVEN_COLORS_FOUND:
		barBlack->setEnabled(true);
		barPhotoCyan->setEnabled(true);
		barPhotoMagenta->setEnabled(true);
		barPhotoYellow->setEnabled(true);
		barCyan->setEnabled(true);
		barMagenta->setEnabled(true);
		barYellow->setEnabled(true);
		barBlack->setProgress(level->black);
		barPhotoCyan->setProgress(level->photocyan);
		barPhotoMagenta->setProgress(level->photomagenta);
		barPhotoYellow->setProgress(level->photoyellow);
		barCyan->setProgress(level->cyan);
		barMagenta->setProgress(level->magenta);
		barYellow->setProgress(level->yellow);
    break;
  }

	delete level;

}


/*!
    \fn KInk::initializeUI(void)
 */
void KInk::initializeUI(void)
{
	QString devName;

	for(int i=0; i<MAX_PARPORT; i++)
	{
		devName = "/dev/lp" + QString::number(i) + " (parport" + QString::number(i) + ")";
		cmbDevices->insertItem(devName);
	}

	for(int i=0; i<MAX_USB_PORT; i++)
	{
		devName = "/dev/usb/lp" + QString::number(i) + " (usb" + QString::number(i) + ")";
		cmbDevices->insertItem(devName);
	}

	cmbDevices->setCurrentItem(currDeviceIdx);
	update();
}


/*!
    \fn KInk::showError(QString errStr)
 */
void KInk::showError(QString errStr)
{
	lblErrorMsg->setText(errStr);
	frmErrorMsg->show();
}

